
#ifndef __SCM_DRIVERS_H_
#define __SCM_DRIVERS_H_

/**
 * @file: scmDrivers.h
 *
 * Tento soubor obsahuje definice pro modul implementujici operace nad
 * ovladaci jadra pomoci rozhrani spravce sluzeb (Service Control Manager).
 */

#include <windows.h>



// Popis vyznamu jednotlivych rutin najdete u jejich implementace v
// souboru scmDrivers.c.

BOOL scmInstallDriver (IN PWCHAR DriverName, IN PWCHAR FileName);
BOOL scmLoadDriver (IN PWCHAR DriverName);
BOOL scmUnloadDriver (IN PWCHAR DriverName);
BOOL scmUninstallDriver (IN PWCHAR DriverName);
BOOL scmServiceExists(IN PWCHAR ServiceName);

BOOL scmInit(VOID);
VOID scmFinit(VOID);



#endif
